package oracle.toplink.essentials.platform.database;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import javax.persistence.GenerationType;

import oracle.toplink.essentials.exceptions.ValidationException;
import oracle.toplink.essentials.platform.database.HSQLPlatform;
import oracle.toplink.essentials.queryframework.ValueReadQuery;

/**
 * HSQLPlatform implementation that may be used with HSQL version 1.8.
 * 
 * This implementation supports the use of Identity columns
 *  
 * 
 * It's likely that this implementation may also be used with version 1.7.2, as this was
 * the version where the Identity column was introduced. It has, however, not been tested
 * with HSQL 1.7.2
 * 
 * @see GenerationType#IDENTITY
 * 
 * @author Morten Andersen-Gott (Accenture)
 *
 */
public class HSQL18IdentityPlatform extends HSQLPlatform {

	@Override
	public ValueReadQuery buildSelectQueryForNativeSequence() {
        ValueReadQuery selectQuery = new ValueReadQuery();
        StringWriter writer = new StringWriter();
        writer.write("CALL IDENTITY()");
        selectQuery.setSQLString(writer.toString());
        return selectQuery;
	}
	
    /**
     * INTERNAL:
     * HSQL 1.8 supports both Sequences and Identity columns
     * This implementation supports sequence numbers through use of an Identity column
     */
	@Override
    public boolean supportsNativeSequenceNumbers() {
        return true;
    }	
	
	/** {@inheritDoc}*/
    public boolean shouldNativeSequenceAcquireValueAfterInsert() {
        return true;
    }
    
    /**
     * Append the receiver's field 'identity' constraint clause to a writer.
     * 
     * The implementation override the default start id in HSQL, 0.
     * JPA applications will typically assume that objects with id equal to 0
     * are new, and have never been persisted. Using 1 as first id to avoid this.
     * 
     */
    public void printFieldIdentityClause(Writer writer) throws ValidationException {
     try {
        //writer.write(" IDENTITY");
       writer.write(" GENERATED BY DEFAULT AS IDENTITY(START WITH 1)");
    } catch (IOException ioException) {
        throw ValidationException.fileError(ioException);
    }
    }    
}
