package oracle.toplink.essentials.tools.schemaframework;

import java.io.IOException;
import java.io.Writer;

import oracle.toplink.essentials.exceptions.DatabaseException;
import oracle.toplink.essentials.exceptions.ValidationException;
import oracle.toplink.essentials.internal.sessions.AbstractSession;
import oracle.toplink.essentials.tools.schemaframework.SchemaManager;
import oracle.toplink.essentials.tools.schemaframework.SequenceDefinition;

/**
 * HSQL Sequence Definition used by schema generation for HSQL Database.
 * 
 * Hasn't been verified as it must be hooked into the code in 
 * 
 * {@link SchemaManager#buildSequenceDefinition} (protected method)
 * 
 * @author Morten Andersen-Gott
 *
 */
public class HSQLSequenceDefinition extends SequenceDefinition{

    /** The increment can be used to support pre-allocation. */
    protected int increment;

    /** The start is the first sequence value that will be available for TopLink to use. */
    protected int start = 1;	
	
	public HSQLSequenceDefinition(String name) {
		super(name);
	}
	
	   public HSQLSequenceDefinition(String name, int preallocationSize) {
	        super(name);
	        setIncrement(preallocationSize);
	    }

	    public HSQLSequenceDefinition(String name, int preallocationSize, int start) {
	        super(name);
	        // sequence value should be positive
	        if(start <= 0) {
	            start = 1;
	        }
	        setIncrement(preallocationSize);
	        setStart(start);
		}	
	

	@Override
	public boolean checkIfExist(AbstractSession session) throws DatabaseException {
        try {
            session.executeSelectingCall(new oracle.toplink.essentials.queryframework.SQLCall("SELECT " + getName() + ".NEXTVAL FROM DUAL"));
        } catch (DatabaseException exception) {
            return false;
        }
        return true;
	}

    /**
     * INTERNAL:
     * Return the SQL required to create the Oracle sequence object.
     */
	@Override
    public Writer buildCreationWriter(AbstractSession session, Writer writer) {
        try {
            writer.write("CREATE SEQUENCE ");
            writer.write(getFullName());
            
            // startWith value calculated using the initial value and increment.
            // The first time TopLink calls select nextval, the value equal to startWith is returned.
            // The first sequence value TopLink may assign is startWith - getIncrement() + 1.
            int startWith = getStart() + getIncrement() - 1;
            writer.write(" START WITH " + startWith);            
            
            if (getIncrement() != 1) {
                writer.write(" INCREMENT BY " + getIncrement());
            }
        } catch (IOException ioException) {
            throw ValidationException.fileError(ioException);
        }
        return writer;
    }

	@Override
	public Writer buildDeletionWriter(AbstractSession session, Writer writer) throws ValidationException {
        try {
            writer.write("DROP SEQUENCE ");
            writer.write(getFullName());
        } catch (IOException ioException) {
            throw ValidationException.fileError(ioException);
        }
        return writer;
	}
	
    public Writer buildAlterIncrementWriter(AbstractSession session, Writer writer) {
        try {
            writer.write("ALTER SEQUENCE ");
            writer.write(getFullName());
            writer.write(" RESTART WITH " + getStart());
        } catch (IOException ioException) {
            throw ValidationException.fileError(ioException);
        }
        return writer;
    }	

	public int getIncrement() {
		return increment;
	}

	public void setIncrement(int increment) {
		this.increment = increment;
	}

	public int getStart() {
		return start;
	}

	public void setStart(int start) {
		this.start = start;
	}

}
